/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;


import java.util.Locale;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationSet;
import android.view.animation.LayoutAnimationController;
import android.view.animation.TranslateAnimation;
import android.webkit.WebView;
import android.widget.LinearLayout;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.base.AbstractActivity;

/**
 * About "Muspy for Android". It shows an html file in a WebView.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class AboutActivity extends AbstractActivity
{
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.about);
		WebView webView = (WebView) findViewById(R.id.webViewAbout);
		
		String fileName = "about.html";
		if (Locale.getDefault().getLanguage().equals(Constants.LOCALE_ES))
		{
			 fileName = "about_es.html";
		}
		
		webView.loadUrl("file:///android_asset/" + fileName);	
		
		 AnimationSet set = new AnimationSet(true);	    
	     Animation animation = new TranslateAnimation(0.0f, 0.0f,1.0f, 0.0f);
	     animation.setDuration(300);
	     set.addAnimation(animation);
	     LayoutAnimationController controller = new LayoutAnimationController(set, 0.25f);
	      
	     LinearLayout bottombar = (LinearLayout) findViewById(R.id.bottombar);
	     bottombar.setLayoutAnimation(controller);
	     bottombar.startAnimation(animation);
	     bottombar.setVisibility(View.VISIBLE);
	}
	
	/**
	 * android.content.Intent.ACTION_SEND
	 * @since 1.2
	 */
	public void share(View view)
	{
		Intent intent = new Intent(android.content.Intent.ACTION_SEND);

		intent.setType("text/plain");
		intent.putExtra(Intent.EXTRA_TEXT, getText(R.string.promote));
		startActivityForResult(Intent.createChooser(intent, getString(R.string.promote_title)),0);		
	}
	
	/**
	 * Google play
	 * @since 1.2
	 */
	public void rating(View view)
	{
		Intent intent = new Intent(Intent.ACTION_VIEW);
		intent.setData(Uri.parse("market://details?id=com.danielme.muspyforandroid"));
		try
		{
			startActivity(intent);
		}	
		catch (ActivityNotFoundException e)
		{
			Builder builder = createInfoDialog(getString(R.string.noappplay));
	
			builder.setNeutralButton(R.string.ok, null);
			AlertDialog alertDialog = builder.setTitle(getString(R.string.infotitle)).create();
			alertDialog.show();
			dialogStyle(alertDialog);
		}
	}
}