/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.AbsListView;
import android.widget.AbsListView.OnScrollListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Spinner;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.adapters.ReleasesAdapter;
import com.danielme.muspyforandroid.activities.adapters.template.CustomListAdapter;
import com.danielme.muspyforandroid.activities.base.AbstractListActivity;
import com.danielme.muspyforandroid.model.Artist;
import com.danielme.muspyforandroid.model.MusicBrainzID;
import com.danielme.muspyforandroid.model.Release;
import com.danielme.muspyforandroid.services.MuspyClient;
import com.danielme.muspyforandroid.utils.ReleasesHolder;
import com.danielme.muspyforandroid.utils.Utils;

/**
 * EndlessListView.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 *
 */
public class ReleasesActivity extends AbstractListActivity
{

	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */
	
	private Spinner spinnerArtists;
	
	private List<Artist> artists;

	//prevents duplicate calls
	private boolean loading = false;
	
	private boolean showingAllItems = false;
	
	private View footerView;
	
	protected String artistmbid;
	
	private boolean entrypoint;
	
	private LoadNextPage loadNextPage = null;
	
	//prevents reloading when rotating with an artists selected
	private boolean dontreload = false;
	
	private SpinnerHandler spinnerHandler = new SpinnerHandler(this);
	
	private OnCreatehandler onCreatehandler = new OnCreatehandler(this);



	/* ************************************************************ */
	/*							OVERRIDE          					*/
	/* ************************************************************ */

	
  //TODO avoid deprecated when using fragemnts
	@SuppressWarnings("deprecation")
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);		
		
		if (this instanceof ReleasesActivityFromSearch)
		{
			setContentView(R.layout.releases_from_search);
		}
		else
		{
			setContentView(R.layout.releases);
		}
		
		entrypoint = getIntent().getBooleanExtra(Constants.RELEASE_ENTRYPOINT, false);

		markButtonPressed(R.id.releases, true);

		if (getIntent().getExtras() != null)
		{
			artistmbid = getIntent().getExtras().getString(Constants.PARAM_ARTIST);
		}
		spinnerArtists = (Spinner) findViewById(R.id.spinner_artists);
		
		footerView = ((LayoutInflater) this.getSystemService(Context.LAYOUT_INFLATER_SERVICE)).inflate(R.layout.listviewfooter, null, false);
		
		final ReleasesHolder releasesHolder = (ReleasesHolder) getLastNonConfigurationInstance();
		if (checkConnection())
		{
			showProgressDialog(null);
			new Thread()
			{
				public void run()
				{
					try
					{
						List<Release> releases = null;
						if (releasesHolder != null)
						{
							releases = releasesHolder.getReleases();
							artists = releasesHolder.getArtists();
							artistmbid = releasesHolder.getArtist();
							if (Utils.hasText(artistmbid))
							{
								dontreload = true;
							}
						}
						else
						{
							releases = MuspyClient.getInstance().getReleases(ReleasesActivity.this, 0, artistmbid, ReleasesActivity.this.getListSize());
							artists =  MuspyClient.getInstance().getArtists(ReleasesActivity.this);
						}
						showingAllItems = releases.isEmpty() ? true : false;
						
						Message msg = new Message();
						msg.what = 0;
						msg.obj = releases;
						onCreatehandler.sendMessage(msg);
					}
					catch (Exception ex)
					{
						Log.e(ReleasesActivity.class.toString(), ex.getMessage(), ex);
						Message message = new Message();
						message.obj = ex;
						message.what = 1;
						onCreatehandler.sendMessage(message);
					}
				}

			}.start();

		}

		getListView().setOnScrollListener(new OnScrollListener()
		{
			@Override
			public void onScrollStateChanged(AbsListView view, int scrollState)
			{
				//not implemented
			}

			@Override
			public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount)
			{
				if (getCustomListAdapter() != null &&  showLast(firstVisibleItem, visibleItemCount,  totalItemCount) && !showingAllItems 
						&& !loading && !isShowingNoConnection() && !isShowingError())
				{
					if (checkConnection())
					{
						loading = true;
						getListView().addFooterView(footerView);
						loadNextPage = new LoadNextPage();
						loadNextPage.execute("");
					}
					else
					{
						getListView().removeFooterView(footerView);
					}
				}
			}			
			
		});
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id)
	{
		Intent intent = new Intent(this, ReleaseActivity.class);
		intent.putExtra(Constants.RELEASE_SEL, (Release) getCustomListAdapter().getItem(position));
		startActivity(intent);
		overridePendingTransition(R.anim.leftin, R.anim.leftout);
	}

	/**
	 * The only way to go back to signin is by pressing "Sign Out". Releases is
	 * the real entry point.
	 */
	@Override
	public void onBackPressed()
	{
		if (entrypoint)
		{	//clean and beautiful hack ;)
	        moveTaskToBack(true); 		
		}
		else
		{
			super.onBackPressed();
		}
	}
	
	/**
	 * @since 1.2
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Object onRetainNonConfigurationInstance()
	  {
		ReleasesHolder releasesHolder  = null;		

		if (getCustomListAdapter() != null) 
		{
	       releasesHolder = new ReleasesHolder();
	       releasesHolder.setReleases((List<Release>) getCustomListAdapter().getObjects());
	       releasesHolder.setArtists(artists);       
	       releasesHolder.setArtist(((Artist)spinnerArtists.getSelectedItem()).getMbid());
	    }
		if (getProgressDialog() != null && getProgressDialog().isShowing())
		{
			getProgressDialog().dismiss();
		}
		if (loadNextPage != null)
		{
			loadNextPage.cancel(true);
		}
		return releasesHolder;
		
	  }
	/* ************************************************************ */
	/*							PRIVATE          					*/
	/* ************************************************************ */
	
	private class LoadNextPage extends AsyncTask<String, Void, String>
	{
		private List<Release> releases;
		private Exception exc = null;
		

		@Override
		protected String doInBackground(String... arg0)
		{
			String mbid = artistmbid;
			String res = null;			

			if (spinnerArtists != null)
			{
				mbid = ((Artist) artists.get(spinnerArtists.getSelectedItemPosition())).getMbid();
			}
			try
			{	
				releases =  MuspyClient.getInstance().getReleases(ReleasesActivity.this, getCount(), mbid, ReleasesActivity.this.getListSize());
			}
			catch (Exception exc)
			{
				Log.e(ReleasesActivity.class.toString(), exc.getMessage(), exc);
				this.exc = exc;
			}

			return res;
		} 

		@Override
		protected void onPostExecute(String result)
		{	
			if (exc != null)
			{				
				showError(exc);
			}
			else
			{
				showingAllItems = releases.size() == 0 ? true : false;
				appendData(releases);
				updateDisplaying(R.string.display, null);
			}
			
			getListView().removeFooterView(footerView);
			loading = false; 				
		}
	}


	private static class  OnCreatehandler extends Handler
	{		
        private final WeakReference<ReleasesActivity> myActivity;
        
        public OnCreatehandler(ReleasesActivity myActivity)
         {
        	      this.myActivity = new WeakReference<ReleasesActivity>(myActivity);
         }   
        
        private ReleasesActivity getActivity()
        {
        	return myActivity.get();
        }
        
		@SuppressWarnings("unchecked")
		@Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();
			if (msg.what == 1)
			{
				getActivity().showError((Exception) msg.obj);
			}
			else
			{
				//hides spinner
				if (Utils.isEmpty(getActivity().artists) && getActivity().spinnerArtists != null)
				{
					if (getActivity().spinnerArtists != null)
					{
						getActivity().findViewById(R.id.artistsLayout).setVisibility(View.GONE);
						getActivity().spinnerArtists.setVisibility(View.GONE);
					}
					getActivity().findViewById(R.id.noartists).setVisibility(View.VISIBLE);					
				}
				else
				{
					if (getActivity().spinnerArtists != null)
					{
						getActivity().spinnerArtists();
					}
					getActivity().getListView().addFooterView(getActivity().footerView, null, false);
					getActivity().setListAdapter(new CustomListAdapter(getActivity(), R.layout.releaselistview, (List<Release>) msg.obj, ReleasesAdapter.class, getCustomAdapterParams()));
					getActivity().getCustomListAdapter().getCount();
					getActivity().getListView().removeFooterView(getActivity().footerView);					

					getActivity().updateDisplaying(R.string.display, null);
				}

			}
		}

		private Map<String, Boolean> getCustomAdapterParams()
		{
			SharedPreferences defaultSharedPreferences = PreferenceManager.getDefaultSharedPreferences(getActivity());

			Boolean b = defaultSharedPreferences.getBoolean(Constants.PREF_LOADCOVER, true);
			Map<String, Boolean> params = new HashMap<String, Boolean>(1);
			params.put(Constants.PREF_LOADCOVER, b);
			return params;
		}

	};


	private static class SpinnerHandler extends Handler
	{
		
		 private final WeakReference<ReleasesActivity> myActivity;
	        
	        public SpinnerHandler(ReleasesActivity myActivity)
	         {
	        	      this.myActivity = new WeakReference<ReleasesActivity>(myActivity);
	         }   
	        
	        private ReleasesActivity getActivity()
	        {
	        	return myActivity.get();
	        }
	        
		@Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();
			if (msg.what == 1)
			{
				getActivity().showError((Exception) msg.obj);
			}
			else
			{
				getActivity().clearData();
				getActivity().appendData((Collection<?>) msg.obj);

				getActivity().updateDisplaying(R.string.display, null);
				// View footerView = ((LayoutInflater)
				// ReleasesActivity.this.getSystemService(Context.LAYOUT_INFLATER_SERVICE)).inflate(R.layout.listviewfooter,
				// null, false);
				// ReleasesActivity.this.getListView().addFooterView(footerView);

				getActivity().getListView().setSelection(0);
			}
		}
	};


	/**
	 * Construye el spinner con los artistas, y añade un listener para que al cambiar la selección se recarge el ListView.
	 */
	private void spinnerArtists()
	{ 
		findViewById(R.id.artistsLayout).setVisibility(View.VISIBLE);
		spinnerArtists.setVisibility(View.VISIBLE);		

		Artist artistDummy = new Artist();
		List<Artist> artistsSpinner = new ArrayList<Artist>(artists.size());
		if (!dontreload)
		{	artistDummy.setName(getString(R.string.all));
			artistDummy.setMbid("");
			artistsSpinner.add(0, artistDummy);
		}
		artistsSpinner.addAll(artists);
		
		artists = artistsSpinner;

		ArrayAdapter<Artist> spinnerAdapter = new ArrayAdapter<Artist>(this, R.layout.spinner, artists);
		spinnerArtists.setAdapter(spinnerAdapter);
		
		if (artistmbid != null)
		{
			spinnerArtists.setSelection(getPositionByMBID(artists, artistmbid));
			artistmbid = null;
		}

		spinnerArtists.setOnItemSelectedListener(new OnItemSelectedListener()
		{
			private boolean first = true;

			@Override
			public void onItemSelected(AdapterView<?> parentView, View selectedItemView, final int position, long id)
			{
				if (dontreload)
				{
					dontreload = false;
				}
				else if (!first && checkConnection())
				{
					showProgressDialog(null);
					new Thread()
					{
						public void run()
						{
							try
							{
								List<Release> releases = MuspyClient.getInstance().getReleases(ReleasesActivity.this, 0, artists.get(position).getMbid(), ReleasesActivity.this.getListSize());
								showingAllItems = releases.size() == 0 ? true : false;
								Message msg = new Message();
								msg.what = 0;
								msg.obj = releases;
								spinnerHandler.sendMessage(msg);
							}
							catch (Exception ex)
							{
								Log.e(ReleasesActivity.class.toString(), ex.getMessage(), ex);
								Message message = new Message();
								message.obj = ex;
								message.what = 1;
								spinnerHandler.sendMessage(message);
							}
						}
					}.start();
				}
				else
				{
					first = false;
				}

			}

			@Override
			public void onNothingSelected(AdapterView<?> parentView)
			{
				// not implemented
			}

		});

	}


	private int getPositionByMBID(List<?> list, String mbid)
	{
		@SuppressWarnings("unchecked")
		ListIterator<MusicBrainzID> listIterator = (ListIterator<MusicBrainzID>) list.listIterator();
		boolean b = false;

		while (listIterator.hasNext() && !b)
		{
			if (listIterator.next().getMbid().equals(mbid))
			{
				b = true;
			}
		}

		if (b)
		{
			return  listIterator.nextIndex() - 1;
		}
		else
		{
			return -1;
		}

	}

}