/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;

import java.lang.ref.WeakReference;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.ImageButton;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.services.MuspyClient;

/**
 * Shows the releases of an artist selected in the search screen.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class ReleasesActivityFromSearch extends ReleasesActivity
{

	private String artistName;
	
	private AddHandler addHandler = new AddHandler(this);
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		artistName = getIntent().getExtras().getString(Constants.PARAM_ARTIST_NAME);
	}
	
	public void pressAdd(View view)
	{
		ImageButton imageButton = (ImageButton) findViewById(R.id.addButton);
		imageButton.setPressed(true);
		imageButton.performClick();
	}
	
	public void add(View view)
	{
		if (checkConnection())
		{
			showProgressDialog(null);
			new Thread()
			{
				public void run()
				{
					try
					{
						MuspyClient.getInstance().addArtist(ReleasesActivityFromSearch.this, artistmbid);
						addHandler.sendEmptyMessage(0);
					}
					catch (Exception ex)
					{
						Log.e(ReleasesActivity.class.toString(), ex.getMessage(), ex);
						Message message = new Message();
						message.obj = ex;
						message.what = 1;
						addHandler.sendMessage(message);
					}
				}

			}.start();
		}	
	}
			
			private static class AddHandler extends Handler
			{
				
				private final WeakReference<ReleasesActivityFromSearch> myActivity;
		        
		        public AddHandler(ReleasesActivityFromSearch myActivity)
		         {
		        	      this.myActivity = new WeakReference<ReleasesActivityFromSearch>(myActivity);
		         }   
		        
		        private ReleasesActivityFromSearch getActivity()
		        {
		        	return myActivity.get();
		        }
		        
				@Override
				public void handleMessage(Message msg)
				{
					getActivity().getProgressDialog().dismiss();
					if (msg.what == 1)
					{
						getActivity().showError((Exception) msg.obj);
					}
					else
					{						
						String text = String.format(getActivity().getString(R.string.toastadded), getActivity().artistName);
						getActivity().showToastSuccess(text);
						Intent intent = new Intent(getActivity(), MyArtistsActivity.class);
						getActivity().defaultTransition(intent);
					}
				}

			};	
	
}