/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;

import java.lang.ref.WeakReference;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.util.Patterns;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.base.AbstractActivity;
import com.danielme.muspyforandroid.services.MuspyClient;
import com.danielme.muspyforandroid.utils.Utils;

/**
 *
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class ResetPasswordActivity extends AbstractActivity
{

	private EditText emailEditText;
	
	private ResetHandler resetHandler = new ResetHandler(this);

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{	
		super.onCreate(savedInstanceState);
		setContentView(R.layout.resetpass);
		String email = getIntent().getExtras().getString(Constants.PARAM_EMAIL);
		emailEditText = (EditText) findViewById(R.id.email);
		emailEditText.setText(email);
		emailEditText.setSelection(email.length());
		emailEditText.setOnEditorActionListener(new OnEditorActionListener()
		{
			@Override
			public boolean onEditorAction(TextView textView, int actionId, KeyEvent event)
			{
				boolean action = false;
				if (actionId == EditorInfo.IME_ACTION_DONE)
				{
					InputMethodManager inputMethodManager = (InputMethodManager) textView.getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
					inputMethodManager.hideSoftInputFromWindow(textView.getWindowToken(), 0);
					reset(textView);
					action = true;
				}
				return action;
			}
		});
	}

	public void reset(View view)
	{
		emailEditText.setText(emailEditText.getText().toString().trim());

		if (!Utils.hasText(emailEditText))
		{
			emailEditText.setError(getString(R.string.noemail));
			//bug in 4.2
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				emailEditText.requestFocus();
			}
		}
		else if (!Patterns.EMAIL_ADDRESS.matcher(emailEditText.getText().toString()).matches())
		{
			emailEditText.setError(getString(R.string.invalidemail));
			//bug in 4.2
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				emailEditText.requestFocus();
			}
		}
		else if (checkConnection())
		{
			showProgressDialog(null);
			new Thread()
			{
				public void run()
				{
					try
					{ 
						boolean isValid =  MuspyClient.getInstance().resetPassword(emailEditText.getText().toString());
						resetHandler.sendEmptyMessage(isValid ? 1 : 0);
					}
					catch (Exception ex)
					{
						Log.e(ResetPasswordActivity.class.toString(), ex.getMessage(), ex);
						Message message = new Message();
						message.obj = ex;
						message.what = -1;
						resetHandler.sendMessage(message);
					}
				}

			}.start();
		}
	}

	private static class ResetHandler extends Handler
	{
    
		private final WeakReference<ResetPasswordActivity> myActivity;
        
        public ResetHandler(ResetPasswordActivity myActivity)
         {
        	      this.myActivity = new WeakReference<ResetPasswordActivity>(myActivity);
         }   
        
        private ResetPasswordActivity getActivity()
        {
        	return myActivity.get();
        }
		
		
		@Override
		public void handleMessage(Message msg)
		{

			getActivity().getProgressDialog().dismiss();
			if (msg.what == 0)
			{
				getActivity().emailEditText.setError(getActivity().getString(R.string.unknownemail));
				//bug in 4.2
				if (Build.VERSION.SDK_INT == Constants.API_421)
				{
					getActivity().emailEditText.requestFocus();
				}
			}
			else if (msg.what == 1)
			{
				getActivity().emailSent();
			}
			else
			{
				getActivity().showError((Exception) msg.obj);
			}
		}

	};

	/**
	 * Shows dialog.
	 */
	private void emailSent()
	{
		String msg = getString(R.string.emailsentreset);
		msg = String.format(msg, emailEditText.getText().toString());
		Builder builder = createInfoDialog(msg);
		builder.setMessage(msg);
		builder.setNeutralButton(R.string.ok, new OnClickListener()
		{
			@Override
			public void onClick(DialogInterface dialog, int which)
			{
				Intent intent = new Intent(ResetPasswordActivity.this, SignInActivity.class);
				defaultTransition(intent);
				finish();
			}
		});
		
		AlertDialog alertDialog = builder.create();
		alertDialog.show();
		alertDialog.setCancelable(false);
		dialogStyle(alertDialog);
	}
}