/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;

import java.lang.ref.WeakReference;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.util.Patterns;
import android.view.View;
import android.widget.CheckBox;
import android.widget.EditText;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.MuspyApplication;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.base.AbstractActivity;
import com.danielme.muspyforandroid.model.UserSettings;
import com.danielme.muspyforandroid.services.MuspyClient;
import com.danielme.muspyforandroid.utils.Utils;

/**
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class SettingsActivity extends AbstractActivity
{
	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */

	private EditText emailEditText;
	//private EditText editTextPass;
	private CheckBox checkBoxNotifications;
	private CheckBox checkBoxAlbum;
	private CheckBox checkBoxSingle;
	private CheckBox checkBoxEP;
	private CheckBox checkBoxCompilation;
	private CheckBox checkBoxLive;
	private CheckBox checkBoxRemix;
	private CheckBox checkBoxOther;
	//private EditText editTextRepeatPass;
	
	private AlertDialog alertDialog;
	
	private ActivityHandler activityHandler = new ActivityHandler(this);
	
	private static final int WHAT_REFRESH = 0;	
	private static final int WHAT_EXC = 1;	
	private static final int WHAT_SAVE = 2;	
	private static final int WHAT_DELETE = 3;


	/* ************************************************************ */
	/*							OVERRIDE           					*/
	/* ************************************************************ */
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{ 
		super.onCreate(savedInstanceState);
		setContentView(R.layout.settings);

		markButtonPressed(R.id.settings, false);
		emailEditText = (EditText) findViewById(R.id.email);
		
		/*editTextPass = (EditText) findViewById(R.id.password);
		editTextRepeatPass = (EditText) findViewById(R.id.repeatpassword);
		editTextRepeatPass.setOnEditorActionListener(new OnEditorActionListener()
		{
			@Override
		public boolean onEditorAction(TextView textView, int actionId, KeyEvent event)
			{
				if (actionId == EditorInfo.IME_ACTION_DONE)
				{
					InputMethodManager imm = (InputMethodManager) textView.getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
					imm.hideSoftInputFromWindow(textView.getWindowToken(), 0);
					return true;
				}
				return false;
			}
		});*/

		checkBoxNotifications = (CheckBox) findViewById(R.id.checkBoxNotifications);
		checkBoxAlbum = (CheckBox) findViewById(R.id.checkBoxAlbum);
		checkBoxSingle = (CheckBox) findViewById(R.id.checkBoxSingle);
		checkBoxEP = (CheckBox) findViewById(R.id.checkBoxEP);
		checkBoxCompilation = (CheckBox) findViewById(R.id.checkBoxCompilation);
		checkBoxLive = (CheckBox) findViewById(R.id.checkBoxLive);
		checkBoxRemix = (CheckBox) findViewById(R.id.checkBoxRemix);
		checkBoxOther = (CheckBox) findViewById(R.id.checkBoxOther);

		if (savedInstanceState != null)
		{
			emailEditText.setText(savedInstanceState.getString("emailEditText"));
			//editTextPass.setText(savedInstanceState.getString("editTextPass"));
			//editTextRepeatPass.setText(savedInstanceState.getString("editTextRepeatPass"));

			checkBoxNotifications.setChecked(savedInstanceState.getBoolean("checkBoxNotifications"));
			checkBoxAlbum.setChecked(savedInstanceState.getBoolean("checkBoxAlbum"));
			checkBoxSingle.setChecked(savedInstanceState.getBoolean("checkBoxSingle"));
			checkBoxEP.setChecked(savedInstanceState.getBoolean("checkBoxEP"));
			checkBoxCompilation.setChecked(savedInstanceState.getBoolean("checkBoxCompilation"));
			checkBoxLive.setChecked(savedInstanceState.getBoolean("checkBoxLive"));
			checkBoxRemix.setChecked(savedInstanceState.getBoolean("checkBoxRemix"));
			checkBoxOther.setChecked(savedInstanceState.getBoolean("checkBoxOther"));
		}
		else 
		{
			refresh(null);
		}
	}
	
	/**
	 * Prevents "Activity has Leaked..." when orientation changes.
	 *
	 */
	@Override
	protected void onPause()
	{
		super.onPause();
		if (getProgressDialog() != null)
		{
			getProgressDialog().dismiss();
		}
		if (alertDialog != null && alertDialog.isShowing())
		{
			alertDialog.dismiss();
		}
		if (getNoConnectionDialog() != null && getNoConnectionDialog().isShowing())
		{
			getNoConnectionDialog().dismiss();
		}
	}
	
	//saves form data when rotating
	@Override
    public void onSaveInstanceState(Bundle outState) 
	{
		super.onSaveInstanceState(outState);
		outState.putString("emailEditText", emailEditText.getText().toString());
		//outState.putString("editTextPass", editTextPass.getText().toString());
		//outState.putString("editTextRepeatPass", editTextRepeatPass.getText().toString());
		outState.putBoolean("checkBoxNotifications", checkBoxNotifications.isChecked());
		outState.putBoolean("checkBoxAlbum", checkBoxAlbum.isChecked());
		outState.putBoolean("checkBoxSingle", checkBoxSingle.isChecked());
		outState.putBoolean("checkBoxEP", checkBoxEP.isChecked());
		outState.putBoolean("checkBoxCompilation", checkBoxCompilation.isChecked());
		outState.putBoolean("checkBoxLive", checkBoxLive.isChecked());
		outState.putBoolean("checkBoxRemix", checkBoxRemix.isChecked());
		outState.putBoolean("checkBoxOther", checkBoxOther.isChecked());
    }


	/* ************************************************************ */
	/*							BUTTONS           					*/
	/* ************************************************************ */

	public void refresh(View view)
	{
		if (checkConnection())
		{
			try
			{
				showProgressDialog(null);
				new Thread()
				{
					public void run()
					{
						try
						{
							UserSettings userSettings =  MuspyClient.getInstance().getUserSettings(getApplicationContext());
							Message message = new Message();
							message.obj = userSettings;
							message.what = WHAT_REFRESH;
							activityHandler.sendMessage(message);
						}
						catch (Exception ex)
						{
							Log.e(SettingsActivity.class.toString(), ex.getMessage(), ex);
							Message message = new Message();
							message.obj = ex;
							message.what = WHAT_EXC;
							activityHandler.sendMessage(message);
						}
					}
				}.start();

			}
			catch (Exception ex)
			{
				Log.e(SettingsActivity.class.toString(), ex.getMessage(), ex);
				showError(ex);
			}
		}

	}

	public void save(View view) throws Exception
	{
		emailEditText.setText(emailEditText.getText().toString().trim());

			if (validate() && checkConnection())
			{
				try
				{
					showProgressDialog(getString(R.string.sendingdata));
					new Thread()
					{
						public void run()
						{
							try
							{
								//TODO password
								boolean noDuplicado =  MuspyClient.getInstance().updateUserSettings(getApplicationContext(), emailEditText.getText().toString(), "", checkBoxNotifications.isChecked(),
										checkBoxAlbum.isChecked(), checkBoxSingle.isChecked(), checkBoxEP.isChecked(), checkBoxCompilation.isChecked(), checkBoxLive.isChecked(), checkBoxRemix.isChecked(),
										checkBoxOther.isChecked());
								Message message = new Message();
								message.obj = noDuplicado;
								message.what = WHAT_SAVE;
								activityHandler.sendMessage(message);
							}
							catch (Exception ex)
							{
								Log.e(SettingsActivity.class.toString(), ex.getMessage(), ex);
								Message message = new Message();
								message.obj = ex;
								message.what = WHAT_EXC;
								activityHandler.sendMessage(message);
							}

						}

					}.start();
				}
				catch (Exception ex)
				{
					Log.e(SettingsActivity.class.toString(), ex.getMessage(), ex);
					showError(ex);
				}

		}
	}
	
	public void checkNotif(View view)
	{
		checkBoxNotifications.setChecked(!checkBoxNotifications.isChecked());
	}
	
	public void delete(View view)
	{
		if (checkConnection())
		{
			try
			{
				showProgressDialog(getString(R.string.sendingdata));
				new Thread()
				{
					public void run()
					{
						try
						{
							MuspyClient.getInstance().deleteAccount(SettingsActivity.this);							
							activityHandler.sendEmptyMessage(WHAT_DELETE);
						}
						catch (Exception ex)
						{
							Log.e(SettingsActivity.class.toString(), ex.getMessage(), ex);
							Message message = new Message();
							message.obj = ex;
							message.what = WHAT_EXC;
							activityHandler.sendMessage(message);
						}
					}
				}.start();

			}
			catch (Exception ex)
			{
				Log.e(SettingsActivity.class.toString(), ex.getMessage(), ex);
				showError(ex);
			}
		}		
	}
	
	public void deleteAccount(View view)
	{		
		Builder builder = createInfoDialog(getString(R.string.deleteaccountdialog));
					
		builder.setCancelable(false).setPositiveButton(R.string.yes, new DialogInterface.OnClickListener()
		{
			public void onClick(DialogInterface dialog, int id)
			{
					SettingsActivity.this.delete(null);
			}
		}).setNegativeButton(R.string.no, new DialogInterface.OnClickListener()
		{
			public void onClick(DialogInterface dialog, int id)
			{
					dialog.cancel();
			}
		});
			
		alertDialog = builder.create();
		alertDialog.show();
		alertDialog.setCancelable(false);
		dialogStyle(alertDialog);		
	}


	/* ************************************************************ */
	/*							PRIVATE           					*/
	/* ************************************************************ */

	private static class ActivityHandler extends Handler 
	{
		private final WeakReference<SettingsActivity> myActivity;
        
        public ActivityHandler(SettingsActivity myActivity)
         {
        	      this.myActivity = new WeakReference<SettingsActivity>(myActivity);
         }   
        
        private SettingsActivity getActivity()
        {
        	return myActivity.get();
        }
		
        @Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();
			if (msg.what == WHAT_EXC)
			{
				getActivity().showError((Exception) msg.obj);
			}
			//loads data
			else if (msg.what == WHAT_REFRESH)
			{				
				populateForm((UserSettings) msg.obj);

			}
			else if (msg.what == WHAT_DELETE)
			{
				getActivity().showToast(R.string.deleteAccountSuccess);
				getActivity().signout(null);
			}
			//saved
			else
			{
				if (!(Boolean) msg.obj)
				{
					getActivity().emailEditText.setError(getActivity().getString(R.string.emailinuse));
					//bug in 4.2
					if (Build.VERSION.SDK_INT == Constants.API_421)
					{
						getActivity().emailEditText.requestFocus();
					}
				}
				else
				{
					MuspyApplication muspyApplication = (MuspyApplication) getActivity().getApplication();

					// TODO password
					if (!getActivity().emailEditText.getText().toString().equals(muspyApplication.getEmail()))
					{
						SharedPreferences sharedPreferences = getActivity().getSharedPreferences(Constants.PREFS, Context.MODE_PRIVATE);

						muspyApplication.setEmail(getActivity().emailEditText.getText().toString());
						SharedPreferences.Editor editor = sharedPreferences.edit();
						editor.putString(Constants.PREF_EMAIL, getActivity().emailEditText.getText().toString());
						editor.commit();
						
						getActivity().emailSent();
					}
					else
					{
						getActivity().showToast(R.string.saved);
					}
				}

			}
		}

		private void populateForm(UserSettings userSettings)
		{
			getActivity().emailEditText.setText(userSettings.getEmail());
			getActivity().checkBoxNotifications.setChecked(userSettings.isNotifications());
			getActivity().checkBoxAlbum.setChecked(userSettings.isFilterAlbum());
			getActivity().checkBoxSingle.setChecked(userSettings.isFilterSingle());
			getActivity().checkBoxEP.setChecked(userSettings.isFilterEP());
			getActivity().checkBoxCompilation.setChecked(userSettings.isFilterCompilation());
			getActivity().checkBoxLive.setChecked(userSettings.isFilterLive());
			getActivity().checkBoxRemix.setChecked(userSettings.isFilterRemix());
			getActivity().checkBoxOther.setChecked(userSettings.isFilterOther());
			//editTextPass.setText("");
			//editTextRepeatPass.setText("");			
		}
	};
	

	private void emailSent()
	{
		Builder builder = createInfoDialog(getString(R.string.emailsentsignup));
	
		builder.setNeutralButton(R.string.ok, null);	
		
		alertDialog = builder.create();
		alertDialog.show();
		alertDialog.setCancelable(false);
		dialogStyle(alertDialog);
	}


	private boolean validate()
	{
		//TODO password
		emailEditText.setError(null);
		//editTextRepeatPass.setError(null);

		if (!Utils.hasText(emailEditText))
		{
			emailEditText.setError(getString(R.string.noemail));
			//bug in 4.2
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				emailEditText.requestFocus();
			}
		}
		else if (!Patterns.EMAIL_ADDRESS.matcher(emailEditText.getText().toString()).matches())
		{
			emailEditText.setError(getString(R.string.invalidemail));
			//bug in 4.2
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				emailEditText.requestFocus();
			}
		}
		/*if (!editTextPass.getText().toString().equals(editTextRepeatPass.getText().toString()))
		{
		editTextRepeatPass.setError(getString(R.string.passnomatch));
		}*/
		return emailEditText.getError() == null; // && editTextRepeatPass.getError() == null;
	}
	
}