/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;

import java.lang.ref.WeakReference;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.util.Patterns;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.MuspyApplication;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.base.AbstractActivity;
import com.danielme.muspyforandroid.model.UserSettings;
import com.danielme.muspyforandroid.services.MuspyClient;
import com.danielme.muspyforandroid.utils.Utils;

/**
 * New Account.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class SignUpActivity extends AbstractActivity
{

	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */

	private EditText emailEditText;
	private EditText passwordEditText;
	private EditText editTextRepeatPassword;

	private SignHandler signHandler = new SignHandler(this);

	/* ************************************************************ */
	/*							OVERRIDE            			    */
	/* ************************************************************ */

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.signup);
		emailEditText = (EditText) findViewById(R.id.editTextEmail);
		passwordEditText = (EditText) findViewById(R.id.editTextPassword);
		editTextRepeatPassword = (EditText) findViewById(R.id.editTextRepeatPassword);
		editTextRepeatPassword.setOnEditorActionListener(new OnEditorActionListener()
		{
			@Override
			public boolean onEditorAction(TextView textView, int actionId, KeyEvent event)
			{
				if (actionId == EditorInfo.IME_ACTION_DONE)
				{
					InputMethodManager inputMethodManager = (InputMethodManager) textView.getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
					inputMethodManager.hideSoftInputFromWindow(textView.getWindowToken(), 0);
					signup(textView);
					return true;
				}
				return false;
			}
		});
	}

	/* ************************************************************ */
	/*							BUTTONS            					*/
	/* ************************************************************ */

	public void signup(View view)
	{
		emailEditText.setText(emailEditText.getText().toString().trim());

		setErrors();

		if (emailEditText.getError() == null && passwordEditText.getError() == null && editTextRepeatPassword.getError() == null 
				&& checkConnection())
		{			
				showProgressDialog(getString(R.string.sendingdata));
				
					new Thread()
					{
						public void run()
						{
							try
							{
								UserSettings userSettings =   MuspyClient.getInstance().signUp(emailEditText.getText().toString(), passwordEditText.getText().toString());					
								Message message = new Message();
								message.obj = userSettings;
								message.what = 0;
								signHandler.sendMessage(message);
							}
							catch (Exception ex)
							{
								Log.e(ResetPasswordActivity.class.toString(), ex.getMessage(), ex);
								Message message = new Message();
								message.obj = ex;
								message.what = -1;
								signHandler.sendMessage(message);
							}
						}

					}.start();
		}
	}
	private void setErrors()
	{
		passwordEditText.setError(null);
		editTextRepeatPassword.setError(null);
		emailEditText.setError(null);

		if (!Utils.hasText(emailEditText))
		{
			emailEditText.setError(getString(R.string.noemail));
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				emailEditText.requestFocus();
			}
		}
		else if (!Patterns.EMAIL_ADDRESS.matcher(emailEditText.getText().toString()).matches())
		{
			emailEditText.setError(getString(R.string.invalidemail));
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				emailEditText.requestFocus();
			}
		}
		if (!Utils.hasText(passwordEditText))
		{
			passwordEditText.setError(getString(R.string.nopass));
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				passwordEditText.requestFocus();
			}
		}
		else if (passwordEditText.getText().toString().length() < 4)
		{
			passwordEditText.setError(getString(R.string.passwordweak));
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				passwordEditText.requestFocus();
			}
		}
		else if (!passwordEditText.getText().toString().equals(editTextRepeatPassword.getText().toString()))
		{
			editTextRepeatPassword.setError(getString(R.string.passnomatch));
			if (Build.VERSION.SDK_INT == Constants.API_421)
			{
				editTextRepeatPassword.requestFocus();
			}

		}
		
	}
	private static class SignHandler extends Handler
	{
		
		private final WeakReference<SignUpActivity> myActivity;
        
        public SignHandler(SignUpActivity myActivity)
         {
        	      this.myActivity = new WeakReference<SignUpActivity>(myActivity);
         }   
        
        private SignUpActivity getActivity()
        {
        	return myActivity.get();
        }
        
		@Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();
			if (msg.what == 0)
			{
				if (msg.obj == null)
				{
					getActivity().emailEditText.setError(getActivity().getString(R.string.emailinuse));
					if (Build.VERSION.SDK_INT == Constants.API_421)
					{
						getActivity().emailEditText.requestFocus();
					}
					
				}
				else
				{
					UserSettings userSettings = (UserSettings) msg.obj;			
					MuspyApplication muspyApplication =(MuspyApplication) getActivity().getApplicationContext();
					muspyApplication.setCredentials(getActivity().emailEditText.getText().toString(), getActivity().passwordEditText.getText().toString(), userSettings.getId());				
					
					getActivity().emailSent();
				}
			}
			else
			{
				getActivity().showError((Exception) msg.obj);
			}
		}

	};

    /**
     * Account created.
     */
	private void emailSent()
	{
		Builder builder = createInfoDialog(getString(R.string.emailsentsignup));
		builder.setNeutralButton(R.string.ok, new OnClickListener()
		{
			@Override
			public void onClick(DialogInterface dialog, int which)
			{
				Intent intent = new Intent(SignUpActivity.this, ReleasesActivity.class);
				intent.putExtra(Constants.RELEASE_ENTRYPOINT, Boolean.TRUE);
				defaultTransition(intent);
				finish();
			}
		});
		
		AlertDialog alertDialog = builder.create();
		alertDialog.show();
		alertDialog.setCancelable(false);
		dialogStyle(alertDialog);
	}

}