/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.cache;

import android.graphics.drawable.Drawable;

import com.danielme.muspyforandroid.Constants;

/**
 * Simple cache for covers.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public final class CoverMemoryCache
{

	private static final BoundedLruCache<String, Drawable> BOUNDED_LRU_CACHE = new BoundedLruCache<String, Drawable>(Constants.CACHE_COVER_MIN, Constants.CACHE_COVER_MAX, 0.75F);

	
	private CoverMemoryCache()
	{
		// not implemented
	}

	public static synchronized void add(String key, Drawable object)
	{
		BOUNDED_LRU_CACHE.put(key, object);
	}

	public static Drawable get(String key)
	{
		return BOUNDED_LRU_CACHE.get(key);
	}	
	
	public static void clearAll()
	{
		BOUNDED_LRU_CACHE.clear();
	}

}