/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.homescreenwidget;

import java.util.List;

import android.app.IntentService;
import android.app.PendingIntent;
import android.appwidget.AppWidgetManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.IBinder;
import android.util.Log;
import android.view.View;
import android.widget.RemoteViews;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.ReleaseActivity;
import com.danielme.muspyforandroid.activities.SignInActivity;
import com.danielme.muspyforandroid.model.Release;
import com.danielme.muspyforandroid.services.MuspyClient;



/**
 * IntentService --> asynchronous
 * @author Daniel Medina (danielme.com)
 * @since 1.1
 *
 */
public class WidgetIntentService extends IntentService 
{

  public WidgetIntentService(String name)
	{
		super(name);
		
	}
  
  public WidgetIntentService()
  {
	super("WidgetIntentService");
  }

  @Override
  public IBinder onBind(Intent intent) 
  {
    return null;
  }

  @Override
  protected void onHandleIntent(Intent intent)
  {
	  
	 AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(this.getApplicationContext());

	 int appWidgetId = intent.getIntExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, 0);
	  
	 RemoteViews remoteView = new RemoteViews(getApplicationContext().getPackageName(),  R.layout.widget);
		
	 ConnectivityManager connectivityManager = (ConnectivityManager) getApplicationContext().getSystemService(Context.CONNECTIVITY_SERVICE);
	 NetworkInfo info = connectivityManager.getActiveNetworkInfo();
	 
	 SharedPreferences sharedPreferences = getApplicationContext().getSharedPreferences(Constants.PREFS, Context.MODE_PRIVATE);
	 String userID = sharedPreferences.getString(Constants.PREF_USERID, null);
	 
	 if (info == null || !info.isConnected() || !info.isAvailable())
	 {
		showMessageWidget(remoteView, R.string.noconnection, getApplicationContext());
	 }
	 else if(userID == null)
	 {
		showMessageWidget(remoteView, R.string.nocredentials, getApplicationContext());				
	 }
	 else
	 {	
	      
		List<Release> releases = null;
		try
		{		
			releases = MuspyClient.getInstance().getReleases(getApplicationContext(), 0, null, 4);					

			if (releases.size() >0 )
			{
				remoteView.setViewVisibility(R.id.dataLayout, View.VISIBLE);
				remoteView.setViewVisibility(R.id.textViewMessage, View.GONE);
					
				remoteView.setTextViewText(R.id.textViewDate1, releases.get(0).getDateLocale());
				remoteView.setTextViewText(R.id.textViewArtist1, releases.get(0).getArtist().getName());
				remoteView.setTextViewText(R.id.textViewRelease1, releases.get(0).getName());
				
				//release details
				Intent intentRelease1 = new Intent(getApplicationContext(), ReleaseActivity.class);
				intentRelease1.putExtra(Constants.RELEASE_SEL, (Release) releases.get(0));
				remoteView.setOnClickPendingIntent(R.id.dataLayout1, PendingIntent.getActivity(getApplicationContext(), 0, intentRelease1, PendingIntent.FLAG_UPDATE_CURRENT));
					
				if (releases.size()>1)
				{
					remoteView.setTextViewText(R.id.textViewDate2, releases.get(1).getDateLocale());
					remoteView.setTextViewText(R.id.textViewArtist2, releases.get(1).getArtist().getName());
					remoteView.setTextViewText(R.id.textViewRelease2, releases.get(1).getName());
					
					Intent intentRelease2 = new Intent(getApplicationContext(), ReleaseActivity.class);
					intentRelease2.putExtra(Constants.RELEASE_SEL, (Release) releases.get(1));
					remoteView.setOnClickPendingIntent(R.id.dataLayout2, PendingIntent.getActivity(getApplicationContext(), 1, intentRelease2, PendingIntent.FLAG_UPDATE_CURRENT));
					
					if (releases.size()>2)
					{
						remoteView.setTextViewText(R.id.textViewDate3, releases.get(2).getDateLocale());
						remoteView.setTextViewText(R.id.textViewArtist3, releases.get(2).getArtist().getName());
						remoteView.setTextViewText(R.id.textViewRelease3, releases.get(2).getName());
						
						Intent intentRelease3 = new Intent(getApplicationContext(), ReleaseActivity.class);
						intentRelease3.putExtra(Constants.RELEASE_SEL, (Release) releases.get(2));
						remoteView.setOnClickPendingIntent(R.id.dataLayout3, PendingIntent.getActivity(getApplicationContext(), 2, intentRelease3, PendingIntent.FLAG_UPDATE_CURRENT));
						//4th release only available on tablet - landscape
						if (releases.size()>3)
						{
							remoteView.setTextViewText(R.id.textViewDate4, releases.get(3).getDateLocale());
							remoteView.setTextViewText(R.id.textViewArtist4, releases.get(3).getArtist().getName());
							remoteView.setTextViewText(R.id.textViewRelease4, releases.get(3).getName());
							
							Intent intentRelease4 = new Intent(getApplicationContext(), ReleaseActivity.class);
							intentRelease4.putExtra(Constants.RELEASE_SEL, (Release) releases.get(3));
							remoteView.setOnClickPendingIntent(R.id.dataLayout4, PendingIntent.getActivity(getApplicationContext(), 3, intentRelease3, PendingIntent.FLAG_UPDATE_CURRENT));
						}
					}
				}		
									
			}
			else
			{
				showMessageWidget(remoteView, R.string.nodata, getApplicationContext());	
			}
							
		} 
		catch (Exception e)
		{
			Log.e("muspy widget", e.getMessage(), e);
			showMessageWidget(remoteView, R.string.exception, getApplicationContext());	
		}	
	}
		
		
	//refresh button
	Intent intentRefresh = new Intent(Constants.INTENT_WIDGET);
	intentRefresh.putExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, appWidgetId);
	PendingIntent pendingIntent = PendingIntent.getBroadcast(getApplicationContext(), appWidgetId, intentRefresh, PendingIntent.FLAG_UPDATE_CURRENT);
	remoteView.setOnClickPendingIntent(R.id.buttonRefresh, pendingIntent);
		
	//open muspy for android
	Intent intentRelease = new Intent(getApplicationContext(), SignInActivity.class);
	remoteView.setOnClickPendingIntent(R.id.buttonAll, PendingIntent.getActivity(getApplicationContext(), 0, intentRelease, 0));
	appWidgetManager.updateAppWidget(appWidgetId, remoteView);
	    
	stopSelf();	
  }

	private void showMessageWidget(RemoteViews remoteView, int id, Context context)
	{
		remoteView.setTextViewText(R.id.textViewMessage, context.getString(id));				
		remoteView.setViewVisibility(R.id.dataLayout, View.INVISIBLE);
		remoteView.setViewVisibility(R.id.textViewMessage, View.VISIBLE);		
	}
  
} 