/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.model;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * 
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class Artist extends MusicBrainzID implements Parcelable
{
	private String name;
	private String disambiguation;
	private String sortname;
	private boolean checked = false;
	
	public static final Parcelable.Creator<Artist> CREATOR =
		             new Parcelable.Creator<Artist>()
		 	            {
		 	                @Override
		 	                public Artist createFromParcel(Parcel parcel)
		 	                {
		 	                    return new Artist(parcel);
		 	                }
		 
		 	                @Override
		 	                public Artist[] newArray(int size)
		 	                {
		 	                    return new Artist[size];
		 	                }
		 	        };

	public Artist(Parcel parcel)
	{
		super();	
		setMbid(parcel.readString());
		name = parcel.readString();
		disambiguation = parcel.readString();
		sortname = parcel.readString();
		checked = Boolean.parseBoolean(parcel.readString());		
	}
	
	public Artist()
	{
		super();	
	}

	public boolean isChecked()
	{
		return checked;
	}

	public void setChecked(boolean checked)
	{
		this.checked = checked;
	}

	public String getName()
	{
		return name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public String getDisambiguation()
	{
		return disambiguation;
	}

	public void setDisambiguation(String disambiguation)
	{
		this.disambiguation = disambiguation;
	}

	public String getSortname()
	{
		return sortname;
	}

	public void setSortname(String sortname)
	{
		this.sortname = sortname;
	}

	@Override
	public String toString()
	{
		String name = this.name;
		if (disambiguation != null && !"".equals(disambiguation))
		{
			name+=" ("+ disambiguation +")";
		}
		return name;
	}

	@Override
	public int describeContents()
	{
		return 0;
	}

	@Override
	public void writeToParcel(Parcel parcel, int flags)
	{
	  parcel.writeString(getMbid());
	  parcel.writeString(name);
	  parcel.writeString(disambiguation);
	  parcel.writeString(sortname);
	  parcel.writeString(checked + "");
	}

}